﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Reflection; 

namespace EditFileProperties
{
    public partial class FrmEditFileProps : Form
    {
        #region fields
        string m_strFilter = Properties.Resources.INPUT_FILES_FILTER;
        private DSOFile.OleDocumentPropertiesClass m_oDocument = null;
        private bool m_fOpenedReadOnly = false;
        #endregion //fields

        #region methods
        /// <summary>
        /// Constructor
        /// </summary>
        public FrmEditFileProps()
        {
            InitializeComponent();
            cboxCustType.Items.AddRange(new object[] { 
                Properties.Resources.TYPE_STRING, 
                Properties.Resources.TYPE_LONG, 
                Properties.Resources.TYPE_DOUBLE, 
                Properties.Resources.TYPE_BOOLEAN, 
                Properties.Resources.TYPE_DATE }); 
        }

        /// <summary>
        /// Creates a new DSOFile.OleDocumentPropertiesClass and opens a file choosen by the user. The properties are filled into the dialog and command tabs are enabled...
        /// </summary>
        /// <returns>true - if document was opened, false - otherwise</returns>
        public bool OpenDocumentProperties()
        {
            bool retVal = false; 
            DSOFile.SummaryProperties oSummProps = null;
            OpenFileDialog oDialog = null;
            string sFile = String.Empty;
            bool fOpenReadOnly = false;

            /*
             * Ask the user for an Office OLE Structure Storage file to read
             * the document properties from. You can also select any common file
             * on a Win2K/WinXP NTFS drive, our a non-Office OLE file...
             */ 
            oDialog = new OpenFileDialog();
            oDialog.Filter = m_strFilter;
            oDialog.FilterIndex = 1;
            oDialog.ShowReadOnly = true; 
            oDialog.CheckFileExists = true; 
            if (oDialog.ShowDialog() == DialogResult.Cancel)
            {
                // Nothing to do is user cancels...
                return retVal;

            }

            sFile = oDialog.FileName;
            fOpenReadOnly = oDialog.ReadOnlyChecked;

            /*
             * Create the OleDocumentProperties object and open the file. The
             * dsoOptionOpenReadOnlyIfNoWriteAccess allows us to open the file
             * read/write if we have access, but go ahead and open read-only if
             * we don't. Since viewing properties is main purpose of the sample
             * it is OK for us to fail write access lock on this open...
             */ 
            m_oDocument = new DSOFile.OleDocumentPropertiesClass();
            m_oDocument.Open(sFile, fOpenReadOnly, DSOFile.dsoFileOpenOptions.dsoOptionOpenReadOnlyIfNoWriteAccess);

            // Here we can tell if file was open read-only...
            m_fOpenedReadOnly = m_oDocument.IsReadOnly;

            // Display the basic file info...
            lbFileName.Text = m_oDocument.Name;
            lbFilePath.Text = m_oDocument.Path;
            picIcon.Image = GetIconImageFromDisp(m_oDocument.Icon);

            // Get the SummaryProperties (these are built-in set)...
            oSummProps = m_oDocument.SummaryProperties;

            /*
             * We'll load a few of these properties into text boxes which we
             * can change in this sample. Other properties can be changed
             * as well, but not by this sample...
             */ 
            txtTitle.Text = oSummProps.Title;
            txtAuthor.Text = oSummProps.Author;
            txtSubject.Text = oSummProps.Subject;
            txtCompany.Text = oSummProps.Company;
            txtComments.Text = oSummProps.Comments;

        
            /*
             * Fill in the Summary/Statistics information in the properties list.
             * These properties are the standard Summay and Document OLE Properties...
             */ 
            ListViewItem lvItem;
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_APPLICATION, oSummProps.ApplicationName);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_VERSION, oSummProps.Version);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_CATEGORY, oSummProps.Category);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_KEYWORDS, oSummProps.Keywords);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_MANAGER, oSummProps.Manager);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_LAST_AUTHOR, oSummProps.LastSavedBy);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_WORD_COUNT, oSummProps.WordCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_PAGE_COUNT, oSummProps.PageCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_PARAGRAPH_COUNT, oSummProps.ParagraphCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_LINE_COUNT, oSummProps.LineCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_CHARACTER_COUNT, oSummProps.CharacterCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_CHARACTER_WS_COUNT, oSummProps.CharacterCountWithSpaces.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_BYTE_COUNT, oSummProps.ByteCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_PRESENTATION_FORMAT, oSummProps.PresentationFormat);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_SLIDE_COUNT, oSummProps.SlideCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_NODE_COUNT, oSummProps.NoteCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_HIDDEN_SLIDES, oSummProps.HiddenSlideCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_MULTIMEDIA_CLIPS, oSummProps.MultimediaClipCount.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_DATE_CREATED, oSummProps.DateCreated == null ? String.Empty : DateTime.Parse(oSummProps.DateCreated.ToString()).ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_DATE_LAST_PRENTED, oSummProps.DateLastPrinted == null ? String.Empty : DateTime.Parse(oSummProps.DateLastPrinted.ToString()).ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_DATE_LAST_SAVED, oSummProps.DateLastSaved == null ? String.Empty : DateTime.Parse(oSummProps.DateLastSaved.ToString()).ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_EDITING_TIME, oSummProps.TotalEditTime.ToString());
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_TEMPLATE, oSummProps.Template);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_REVISION, oSummProps.RevisionNumber);
            StatListView.Items.Add(lvItem);
            lvItem = GetLvItemForProperty(Properties.Resources.SUMMARY_IS_SHARED, oSummProps.SharedDocument.ToString());
            StatListView.Items.Add(lvItem);
            
            // Add a few other items that pertain to OLE files only...
            if (m_oDocument.IsOleFile)
            {
                string testVal = String.Empty;
                try
                {
                    testVal = m_oDocument.CLSID;
                }
                catch {}
                lvItem = GetLvItemForProperty("CLSID", testVal);
                StatListView.Items.Add(lvItem);
                testVal = String.Empty;
                try
                {
                    testVal = m_oDocument.ProgID;
                }
                catch {}
                lvItem = GetLvItemForProperty("ProgID", testVal);
                StatListView.Items.Add(lvItem);
                testVal = String.Empty;
                try
                {
                    testVal = m_oDocument.OleDocumentFormat;
                }
                catch {}
                lvItem = GetLvItemForProperty("OleFormat", testVal);
                StatListView.Items.Add(lvItem);
                testVal = String.Empty;
                try
                {
                    testVal = m_oDocument.OleDocumentType;
                }
                catch {}
                lvItem = GetLvItemForProperty("OleType", testVal);
                StatListView.Items.Add(lvItem);
            }

            // Now load the custom properties...
            foreach(DSOFile.CustomProperty oCustProp in m_oDocument.CustomProperties)
            {
                if (oCustProp.Type != DSOFile.dsoFilePropertyType.dsoPropertyTypeUnknown)
                {
                    lvItem = GetLvItemForProperty(
                        oCustProp.Name, 
                        oCustProp.get_Value() == null ? String.Empty : oCustProp.get_Value().ToString(),
                        oCustProp.Type);
                    CustListView.Items.Add(lvItem);
                }
            }

            // Enable/Disable text items if file is open read only...
            EnableItems(!m_fOpenedReadOnly);

            // The operation was successful.
            retVal = true;  
            PropTabs.Enabled = true; 
            return retVal; 
        }

        /// <summary>
        /// Helper function to enable/disable controls with respect to read-only mode for the current file...
        /// </summary>
        /// <param name="bEnable">true - if need set enable, false - for disable</param>
        private void EnableItems(bool bEnable)
        {
            txtTitle.Enabled 
                = txtAuthor.Enabled 
                = txtSubject.Enabled 
                = txtCompany.Enabled
                = txtComments.Enabled 
                = txtCustName.Enabled
                = txtCustValue.Enabled
                = cboxCustType.Enabled 
                = cmdAdd.Enabled = bEnable;
        }

        /// <summary>
        /// Helper function to take PictureDisp and create GDI+ Bitmap Image for the file icon to display in WinForm PictureBox...
        /// </summary>
        /// <param name="oDispPicture">input PictureDisp oblect</param>
        /// <returns>Image object or null</returns>
        private System.Drawing.Image GetIconImageFromDisp(object oDispPicture)
        {
            System.Drawing.Image retval = null;
            long iType = 0;
            int iHandle = 0;
            object[] args = null;
            try
            {
                // Confirm that object contains an ICON picture...
                iType = Int32.Parse(oDispPicture.GetType().InvokeMember(
                    "Type", 
                    BindingFlags.GetProperty, 
                    null, 
                    oDispPicture, 
                    args).ToString());
                if (iType == 3)
                {
                    // So, ask for the handle...
                    iHandle = Int32.Parse(oDispPicture.GetType().InvokeMember(
                        "Handle", 
                        BindingFlags.GetProperty, 
                        null, 
                        oDispPicture, 
                        args).ToString());
                    retval = System.Drawing.Bitmap.FromHicon(new System.IntPtr(iHandle));
                }
            }
            catch { }
            return retval;
        }

        /// <summary>
        /// Helper function to take name and value and return ListViewItem which can be added to the listview for the propset being displayed.
        /// </summary>
        /// <param name="sName">name property</param>
        /// <param name="sValue">value property</param>
        /// <param name="iType">type property</param>
        /// <returns>ListViewItem object</returns>
        private ListViewItem GetLvItemForProperty(string sName, string sValue, DSOFile.dsoFilePropertyType iType = DSOFile.dsoFilePropertyType.dsoPropertyTypeUnknown)
        {
            ListViewItem lvItem = new ListViewItem();
            string sTypeName = String.Empty;  
            lvItem.Text = sName;
            lvItem.SubItems.Add(sValue);
            switch (iType)
            {
                case DSOFile.dsoFilePropertyType.dsoPropertyTypeString:
                    sTypeName = Properties.Resources.TYPE_STRING;
                    break;
                case DSOFile.dsoFilePropertyType.dsoPropertyTypeDouble:
                    sTypeName = Properties.Resources.TYPE_DOUBLE;
                    break;
                case DSOFile.dsoFilePropertyType.dsoPropertyTypeLong:
                    sTypeName = Properties.Resources.TYPE_LONG;
                    break;
                case DSOFile.dsoFilePropertyType.dsoPropertyTypeBool:
                    sTypeName = Properties.Resources.TYPE_BOOLEAN;
                    break;
                case DSOFile.dsoFilePropertyType.dsoPropertyTypeDate:
                    sTypeName = Properties.Resources.TYPE_DATE;
                    break;
            }
            lvItem.SubItems.Add(sTypeName);
            return lvItem;
        }

        /// <summary>
        /// Closed the open document and clears the dialog. Can prompt the user to save the changes made if open read-write mode.
        /// </summary>
        /// <param name="bPromptToSaveIfDirty">true - if need show dialog with confirm to save changes, false - otherwise</param>
        private void CloseCurrentDocument(bool bPromptToSaveIfDirty)
        {
            if (m_oDocument != null)
            {
                // If changes where made, ask user if they want to save...
                if (bPromptToSaveIfDirty && m_oDocument.IsDirty && !m_fOpenedReadOnly)
                {
                    if (MessageBox.Show(
                        this,
                        Properties.Resources.MSG_Q_SAVE_CHANGES,
                        Properties.Resources.CAPTION_CONFIRM, 
                        MessageBoxButtons.YesNo, 
                        MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                    {
                        m_oDocument.Save();
                    }
                }

                // Close before exit...
                m_oDocument.Close();
                m_oDocument = null; 
            }
            ClearControls();
            PropTabs.Enabled = false; 
        }

        /// <summary>
        /// Helper function to restore dialog controls to "blank" slate...
        /// </summary>
        private void ClearControls()
        {
            lbFileName.Text = Properties.Resources.LBL_FILE_NAME;
            lbFilePath.Text 
                = txtTitle.Text 
                = txtAuthor.Text 
                = txtSubject.Text 
                = txtCompany.Text 
                = txtComments.Text 
                = txtCustName.Text 
                = txtCustValue.Text
                = String.Empty;
            cboxCustType.SelectedIndex = 0;
            picIcon.Image = null;
            if (StatListView.Items.Count > 0)
            {
                StatListView.Items.Clear();
            }
            if (CustListView.Items.Count > 0)
            {
                CustListView.Items.Clear();
            }
            cmdRemove.Enabled = false; 
        }
        #endregion //methods

        #region events
        /// <summary>
        /// Event Click for object cmdOpen
        /// </summary>
        /// <param name="sender">object raise event</param>
        /// <param name="e">args of event</param>
        private void cmdOpen_Click(object sender, EventArgs e)
        {
            CloseCurrentDocument(true);
            if (!OpenDocumentProperties())
            {
                CloseCurrentDocument(false);
            }
        }

        /// <summary>
        /// Event Click for object cmdAdd
        /// </summary>
        /// <param name="sender">object raise event</param>
        /// <param name="e">args of event</param>
        private void cmdAdd_Click(object sender, EventArgs e)
        {
            DSOFile.CustomProperty oCustProp = null;
            string sName = null;
            string sValueText = null;
            object vValue = null;

            sName = txtCustName.Text;
            sValueText = txtCustValue.Text;

            // We can't add a custom property unless we have a valid name
            // and value, so we do a quick check now to avoid error later on.
            if (String.IsNullOrEmpty(sName) || String.IsNullOrEmpty(sValueText))
            {
                txtCustName.Select();
                return;
            }

            try
            {
                switch (cboxCustType.SelectedIndex + 1)
                {
                    case 2:
                        vValue = Int32.Parse(sValueText);
                        break;
                    case 3:
                        vValue = Double.Parse(sValueText);
                        break;
                    case 4:
                        vValue = Boolean.Parse(sValueText);
                        break;
                    case 5:
                        vValue = DateTime.Parse(sValueText);
                        break;
                    default :
                        vValue = sValueText;
                        break;
                }

                // Add the property to the custom properties list...
                oCustProp = m_oDocument.CustomProperties.Add(sName, vValue);

                // If that succeeded, add the item to the list view...
                CustListView.Items.Add(GetLvItemForProperty(sName, sValueText, oCustProp.Type));
                txtCustName.Text
                    = txtCustValue.Text
                    = String.Empty;
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    this,
                    Properties.Resources.MSG_ERR_ITEM_NOT_ADDED
                        + Environment.NewLine + Properties.Resources.LBL_ERROR 
                        + " " + ex.Message,
                    Properties.Resources.CAPTION_ERROR,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Event Click for object cmdRemove
        /// </summary>
        /// <param name="sender">object raise event</param>
        /// <param name="e">args of event</param>
        private void cmdRemove_Click(object sender, EventArgs e)
        {
            DSOFile.CustomProperty oRmProp = null;
            int iRemoveCnt = 0;
            int i = 0;
            string sName = String.Empty;

            // Get the count of selected items (normally this is 1)...
            iRemoveCnt = CustListView.SelectedItems.Count;
            if (iRemoveCnt < 1)
            {
                MessageBox.Show(this,
                    Properties.Resources.MSG_ERR_NO_ITEMS_TO_REMOVE,
                    Properties.Resources.CAPTION_WARNING,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning);

                return;
            }

            /*
             * For each fo the selected items, remove them from the document.
             * We loop backwards to not change the list as we also want to 
             * remove the listview item when the property is removed...
             */
            for (i = iRemoveCnt - 1; i > (-1); i--)
            {
                sName = CustListView.SelectedItems[i].Text;
                try
                {
                    oRmProp = m_oDocument.CustomProperties[sName];
                    oRmProp.Remove();
                    oRmProp = null;
                    CustListView.SelectedItems[i].Remove();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(
                        this,
                        Properties.Resources.MSG_ERR_UNABLE_TO_REMOVE1 + " '" + sName
                            + "' " + Properties.Resources.MSG_ERR_UNABLE_TO_REMOVE2 
                            + Environment.NewLine
                            + Properties.Resources.LBL_ERROR + " " + ex.Message,
                        Properties.Resources.CAPTION_ERROR,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
            cmdRemove.Enabled = false;
        }

        /// <summary>
        /// Override event OnClosing
        /// </summary>
        /// <param name="e">arg event</param>
        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);
            CloseCurrentDocument(true);
        }

        /// <summary>
        /// Event SelectedIndexChanged of object CustListView
        /// </summary>
        /// <param name="sender">event owner</param>
        /// <param name="e">event arg</param>
        private void CustListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!m_fOpenedReadOnly)
            {
                cmdRemove.Enabled = (CustListView.SelectedItems.Count > 0);
            }
        }

        /// <summary>
        /// Event Validated of object txtTitle
        /// </summary>
        /// <param name="sender">event owner</param>
        /// <param name="e">event arg</param>
        private void txtTitle_Validated(object sender, EventArgs e)
        {
            if (m_oDocument != null)
            {
                DSOFile.SummaryProperties oSummProps = m_oDocument.SummaryProperties;
                if (txtTitle.Text.CompareTo(oSummProps.Title) != 0)
                {
                    oSummProps.Title = txtTitle.Text;
                }
            }
        }

        /// <summary>
        /// Event Validated of object txtAuthor
        /// </summary>
        /// <param name="sender">event owner</param>
        /// <param name="e">event arg</param>
        private void txtAuthor_Validated(object sender, EventArgs e)
        {
            if (m_oDocument != null)
            {
                DSOFile.SummaryProperties oSummProps = m_oDocument.SummaryProperties;
                if (txtTitle.Text.CompareTo(oSummProps.Author) != 0)
                {
                    oSummProps.Author = txtTitle.Text;
                }
            }
        }

        /// <summary>
        /// Event Validated of object txtSubject
        /// </summary>
        /// <param name="sender">event owner</param>
        /// <param name="e">event arg</param>
        private void txtSubject_Validated(object sender, EventArgs e)
        {
            if (m_oDocument != null)
            {
                DSOFile.SummaryProperties oSummProps = m_oDocument.SummaryProperties;
                if (txtSubject.Text.CompareTo(oSummProps.Subject) != 0)
                {
                    oSummProps.Subject = txtSubject.Text;
                }
            }
        }

        /// <summary>
        /// Event Validated of object txtCompany
        /// </summary>
        /// <param name="sender">event owner</param>
        /// <param name="e">event arg</param>
        private void txtCompany_Validated(object sender, EventArgs e)
        {
            if (m_oDocument != null)
            {
                DSOFile.SummaryProperties oSummProps = m_oDocument.SummaryProperties;
                if (txtCompany.Text.CompareTo(oSummProps.Company) != 0)
                {
                    oSummProps.Company = txtCompany.Text;
                }
            }
        }

        /// <summary>
        /// Event Validated of object txtComments
        /// </summary>
        /// <param name="sender">event owner</param>
        /// <param name="e">event arg</param>
        private void txtComments_Validated(object sender, EventArgs e)
        {
            if (m_oDocument != null)
            {
                DSOFile.SummaryProperties oSummProps = m_oDocument.SummaryProperties;
                if (txtComments.Text.CompareTo(oSummProps.Comments) != 0)
                {
                    oSummProps.Comments = txtComments.Text;
                }
            }
        }
        #endregion //events
    }
}
